package com.nokia.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.nokia.dao.CityDao;
import com.nokia.dao.UserDao;
import com.nokia.dao.UserRoleDao;
import com.nokia.pojo.CityEntity;
import com.nokia.pojo.RoleEntity;
import com.nokia.pojo.UserEntity;
import com.nokia.pojo.UserRoleEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Service
public class UserService extends ServiceImpl<UserDao, UserEntity> {

    private final UserDao userDao;
    private final UserRoleDao userRoleDao;
    private final CityDao cityDao;

    @Autowired
    public UserService(UserDao userDao, UserRoleDao userRoleDao, CityDao cityDao) {
        this.userDao = userDao;
        this.userRoleDao = userRoleDao;
        this.cityDao = cityDao;
    }

    @Transactional
    public String add(UserEntity table2User) {
        UserEntity query = new UserEntity();
        query.setLoginName(table2User.getLoginName());
        UserEntity one = baseMapper.selectOne(Wrappers.query(query));
        if (one != null) {
            return "loginName: " + table2User.getLoginName() + " 已存在";
        } else {
            table2User.setCityId(table2User.getCityId());
            baseMapper.insert(table2User);
            Set<RoleEntity> roleEntityList1 = table2User.getRoles();
            UserRoleEntity userRoleEntity = new UserRoleEntity();
            for (RoleEntity role : roleEntityList1) {
                userRoleEntity.setUserId(table2User.getUserId());
                userRoleEntity.setRoleId(role.getRoleId());
                userRoleDao.insert(userRoleEntity);
            }
        }
        return "ok";
    }

    @Transactional
    public String delete(Integer userId) {
        // UserEntity userEntity = baseMapper.selectById(userId);
        /* cityDao.deleteById(userEntity.getCityId()); */
        QueryWrapper<UserRoleEntity> objectQueryWrapper = new QueryWrapper<>();
        objectQueryWrapper.eq("user_id", userId);
        List<UserRoleEntity> userRoleEntities = userRoleDao.selectList(objectQueryWrapper);
        for (UserRoleEntity userRoleEntity : userRoleEntities) {
            Integer id = userRoleEntity.getId();
            userRoleDao.deleteById(id);
        }
        baseMapper.deleteById(userId);
        return "ok";
    }

    @Transactional
    public String update(UserEntity user) {
        // 更新user表
        baseMapper.updateById(user);
        // 更新user_role 分为2步
        Integer userId = user.getUserId();
        QueryWrapper<UserRoleEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", userId);
        // 1. 删除原user_role
        userRoleDao.delete(queryWrapper);
        // 2. 添加新的user_role
        for (RoleEntity role : user.getRoles()) {
            UserRoleEntity userRoleEntity = new UserRoleEntity();
            userRoleEntity.setUserId(user.getUserId());
            userRoleEntity.setRoleId(role.getRoleId());
            userRoleDao.insert(userRoleEntity);
        }
        return "ok";
    }

    public UserEntity getByLoginName(String loginName) {
        return userDao.getByLoinName(loginName);
    }

    public Map<String, Object> select(Integer userId) {
        Map<String, Object> hashMap = new HashMap<>();
        if (userId == null) {
            hashMap.put("userId", "为空");
            return hashMap;
        }
        UserEntity userEntity = baseMapper.selectById(userId);
        QueryWrapper<UserEntity> userEntityQueryWrapper = new QueryWrapper<>();
        userEntityQueryWrapper.eq("user_id", userId);
        UserEntity one = baseMapper.selectOne(userEntityQueryWrapper);
        if (one == null) {
            hashMap.put("userId", "不正确");
            return hashMap;
        }
        Integer cityId = userEntity.getCityId();
        CityEntity cityEntity = cityDao.selectById(cityId);
        // 他有两个角色 对 我知道 用for遍历来查 不需要for遍历查
        QueryWrapper<UserRoleEntity> objectQueryWrapper = new QueryWrapper<>();
        objectQueryWrapper.eq("user_id", userId);
        List<UserRoleEntity> userRoleEntities = userRoleDao.selectList(objectQueryWrapper);
        hashMap.put("userEntity", userEntity);
        hashMap.put("cityEntity", cityEntity);
        hashMap.put("userRoleEntities", userRoleEntities);
        return hashMap;
    }

}
